@@ -19,6 +19,8 @@ import os
 import progressbar
 import requests
 import threading
+import traceback
+import urllib.parse
 
 # Google API details for a native/installed application for API project
 # grow-prod.
@@ -108,10 +110,11 @@ class HttpWithApiKey(httplib2.Http):
 
 class RpcError(Error):
 
-    def __init__(self, status, message=None, data=None):
+    def __init__(self, status, message=None, data=None, tb=None):
         self.status = status
         self.message = data['error_message'] if data else message
         self.data = data
+        self.tb = tb
 
     def __repr__(self):
         return '{}: {}'.format(self.status, self.message)
@@ -269,17 +272,22 @@ class WebReview(object):
             bar.start()
 
         def _execute_request_signing_request(reqs, errs, service, item):
-            batched_paths_to_contents = dict(item)
-            req = self.gs.create_sign_requests_request(
-                verb, self.fileset, batched_paths_to_contents)
             try:
-                resp = service.sign_requests(body=req).execute()
-            except errors.HttpError as e:
-                errs += [(e.resp.status, e._get_reason().strip())]
-                return
-            if bar and False:
-                bar.update(bar.value + 1)
-            reqs += resp['signed_requests']
+                batched_paths_to_contents = dict(item)
+                req = self.gs.create_sign_requests_request(
+                    verb, self.fileset, batched_paths_to_contents)
+                try:
+                    resp = service.sign_requests(body=req).execute()
+                except errors.HttpError as e:
+                    errs += [(e.resp.status, e._get_reason().strip(), None)]
+                    return
+                if bar:
+                    bar.update(bar.value + 1)
+                reqs += resp['signed_requests']
+            except Exception as err:
+                # Return traceback as a list of strings - complex objects fail.
+                tb = traceback.format_tb(err.__traceback__)
+                errs += [('Error creating signed request', str(err), tb)]
 
         for item in batched_items:
             service = self.get_service()
@@ -288,8 +296,8 @@ class WebReview(object):
         self.pool.close()
         self.pool.join()
         if error_objs:
-            status, reason = error_objs[0]
-            raise WebReviewRpcError(status, reason)
+            status, reason, tb = error_objs[0]
+            raise WebReviewRpcError(status, reason, tb=tb)
         if bar:
             bar.finish()
         return signed_requests
@@ -325,13 +333,13 @@ class WebReview(object):
         resp = None
         try:
             resp = self.gs.execute_signed_request(req, content)
-        except GoogleStorageRpcError as e:
-            error = e
+        except Exception as err:
+            error = err
         with self.lock:
             if resp is not None:
                 resps[path] = resp
             if error is not None:
-                errors[path] = e
+                errors[path] = error
         if bar is not None:
             bar.update(bar.value + 1)
 
@@ -349,8 +357,11 @@ class WebReview(object):
             bar.start()
             for req in signed_requests:
                 path = req['path']
-                args = (req, path, paths_to_rendered_doc[
-                        path].read(), bar, resps, errors)
+                path_raw = urllib.parse.unquote(path)
+                content = paths_to_rendered_doc[path_raw].read()
+                if isinstance(content, str):
+                    content = content.encode('utf-8')
+                args = (req, path, content, bar, resps, errors)
                 self.pool.apply_async(self._execute, args=args)
             self.pool.close()
             self.pool.join()
@@ -358,8 +369,11 @@ class WebReview(object):
         else:
             req = signed_requests[0]
             path = req['path']
-            self._execute(req, path, paths_to_rendered_doc[
-                          path].read(), None, resps, errors)
+            path_raw = urllib.parse.unquote(path)
+            content = paths_to_rendered_doc[path_raw].read()
+            if isinstance(content, str):
+                content = content.encode('utf-8')
+            self._execute(req, path, content, None, resps, errors)
         return resps, errors
 
     @classmethod
@@ -368,12 +382,11 @@ class WebReview(object):
         for pre, _, files in os.walk(build_dir):
             for f in files:
                 path = os.path.join(pre, f)
-                fp = open(path)
-                path = path.replace(build_dir, '')
-                if not path.startswith('/'):
-                    path = '/{}'.format(path)
-                content = fp.read()
-                fp.close()
+                with open(path) as fp:
+                    path = path.replace(build_dir, '')
+                    if not path.startswith('/'):
+                        path = '/{}'.format(path)
+                    content = fp.read()
                 if isinstance(content, str):
                     content = content.encode('utf-8')
                 paths_to_rendered_doc[path] = RenderedDocStub(content=content)
@@ -385,7 +398,7 @@ class GoogleStorageSigner(object):
     @staticmethod
     def create_unsigned_request(verb, path, content=None):
         req = {
-            'path': path,
+            'path': urllib.parse.quote(path),
             'verb': verb,
         }
         if verb == Verb.PUT:
@@ -394,7 +407,16 @@ class GoogleStorageSigner(object):
             else:
                 mimetype = mimetypes.guess_type(path)[0]
                 mimetype = mimetype or 'application/octet-stream'
-            md5_digest = base64.b64encode(hashlib.md5(content).digest())
+            try:
+                content = content.encode('utf-8')
+            except AttributeError:
+                try:
+                    content = content.decode('utf-8')
+                except (AttributeError, UnicodeDecodeError):
+                    pass
+
+            md5_digest = base64.b64encode(
+                hashlib.md5(content).digest()).decode("utf-8")
             req['headers'] = {}
             req['headers']['content_length'] = str(len(content))
             req['headers']['content_md5'] = md5_digest
